% [belH, belV, belD1, belD2] = ...
%   gbpTypicalFast8(potH, potV, potD1, potD2, nIters, nColors)
% 
% run gbp with a grid network having 8-connectivity
% The pair potentials are defined by the ising model, 1 on the diagonal and 
%     the values given in potH, potV, potD1, potD2 on the off-diagonal
% potH - horizontal pairs, pot(y,x) is related to (y,x+1)
% potV - vertical pairs, pot(y,x) is related to (y+1,x)
% potD1 - diagonal pairs, pot(y,x) is related to (y+1,x+1)
% potD2 - diagonal pairs, pot(y,x) is related to (y+1,x-1)
% nIters - number of iterations, typically 10
% nColors - number of states/colors per node, typically 6
%
% belH, belV, belD1, belD2: The beliefs for the corresponding pairs
function [corrH, corrV, corrD1, corrD2] = ...
   gbpTypicalFast8(potH, potV, potD1, potD2, nIters, nColors)

[n,m]=size(potH);
msgsLR=ones(n+2,m+2);
msgsRL=ones(n+2,m+2);
msgsTB=ones(n+2,m+2);
msgsBT=ones(n+2,m+2);
msgsLRn=ones(n+2,m+2);
msgsRLn=ones(n+2,m+2);
msgsTBn=ones(n+2,m+2);
msgsBTn=ones(n+2,m+2);
potH = padOnes(potH);
potV = padOnes(potV);
potD1 = padOnes(potD1);
potD2 = padOnes(potD2);
q=nColors;
for i=1:nIters, 
   msgsLRn(2:end-1,3:end) = ...
       computeMessages8(q, potV(2:end-1,2:end-1).*msgsLR(2:end-1,2:end-1),...   
                          potV(2:end-1,3:end),... 
                          potH(2:end-1,2:end-1).*msgsTB(2:end-1,2:end-1),...   
                          potH(3:end  ,2:end-1).*msgsBT(3:end  ,2:end-1),...
                          potD1(2:end-1,2:end-1), ...
                          potD2(2:end-1,3:end));
                      
   msgsRLn(2:end-1,1:end-2) = ...
       computeMessages8(q, potV(2:end-1,2:end-1).*msgsRL(2:end-1,2:end-1),...   
                          potV(2:end-1,1:end-2),... 
                          potH(2:end-1,1:end-2).*msgsTB(2:end-1,1:end-2),...   
                          potH(3:end  ,1:end-2).*msgsBT(3:end,1:end-2),...
                          potD2(2:end-1,2:end-1), ...
                          potD1(2:end-1,1:end-2));
                          
       
   msgsTBn(3:end,2:end-1) = ...
       computeMessages8(q, potH(2:end-1,2:end-1).*msgsTB(2:end-1,2:end-1),...   
                          potH(3:end,2:end-1),... 
                          potV(2:end-1,2:end-1).*msgsLR(2:end-1,2:end-1),...   
                          potV(2:end-1,3:end).*msgsRL(2:end-1,3:end), ...
                          potD1(2:end-1,2:end-1), ...
                          potD2(2:end-1,3:end));
                      
   msgsBTn(1:end-2,2:end-1) = ...
       computeMessages8(q, potH(2:end-1,2:end-1).*msgsBT(2:end-1,2:end-1),...   
                          potH(1:end-2,2:end-1),... 
                          potV(1:end-2,2:end-1).*msgsLR(1:end-2,2:end-1),...   
                          potV(1:end-2,3:end).*msgsRL(1:end-2,3:end),...
                          potD2(1:end-2,3:end), ...
                          potD1(1:end-2,2:end-1));

   errr = abs(msgsLR-msgsLRn)+abs(msgsRL-msgsRLn)+abs(msgsTB-msgsTBn)+abs(msgsBT-msgsBTn);
   if (sum(errr(:)) < 0.000001)
       break;
   end
   msgsLR=msgsLRn;
   msgsRL=msgsRLn;
   msgsTB=msgsTBn;
   msgsBT=msgsBTn;
end
belV=msgsLR.*msgsRL.*potV;
belV=belV(2:end-1,2:end-1);
belH=msgsTB.*msgsBT.*potH;
belH=belH(2:end-1,2:end-1);

belD1=diag_bel_8( q,...
                  potH(3:end  ,2:end-1).*msgsBT(3:end  ,2:end-1),...
                  potH(2:end-1,2:end-1).*msgsTB(2:end-1,2:end-1),...  
			      potV(2:end-1,2:end-1).*msgsLR(2:end-1,2:end-1),...  
			      potV(2:end-1,3:end  ).*msgsRL(2:end-1,3:end),...   
                  potD2(2:end-1,3:end),...
			      potD1(2:end-1,2:end-1) ...
                  );
			  
belD2=diag_bel_8( q,...
                  potV(2:end-1,1:end-2).*msgsLR(2:end-1,1:end-2),...                   
                  potV(2:end-1,2:end-1 ).*msgsRL(2:end-1,2:end-1),...   
                  potH(2:end-1,1:end-2).*msgsTB(2:end-1,1:end-2),...   
                  potH(3:end  ,1:end-2).*msgsBT(3:end  ,1:end-2),...
                  potD1(2:end-1,1:end-2),...
                  potD2(2:end-1,2:end-1));
			  
corrH=1./(1+(q-1)*belH);
corrV=1./(1+(q-1)*belV);
corrD1=1./(1+(q-1)*belD1);
corrD2=1./(1+(q-1)*belD2);



			  
			  
			  