function [SimHorizontal,SimVertical,SimDiagonal1,SimDiagonal2]=...
	     imageSimAssafStyle2(img,typ,connectivity)
% outputs similarity matrix 


n=size(img,1);
m=size(img,2);

if connectivity==4
	if typ == 'color'
		[SimHorizontal,SimVertical]=run_c4(img,n,m,'inner_sim_col');
	else % gray
		[SimHorizontal,SimVertical]=run_c4(img,n,m,'inner_sim_gray');
    end
    SimDiagonal1=[];
    SimDiagonal2=[];
	
else % connectivity=8
	if typ == 'color'
		[SimHorizontal,SimVertical,SimDiagonal1,SimDiagonal2]=run_c8(img,n,m,'inner_sim_col');
    else % gray
		[SimHorizontal,SimVertical,SimDiagonal1,SimDiagonal2]=run_c8(img,n,m,'inner_sim_gray');
    end  
	
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
%%%%%%%%%% 8 neighbours
function [SimHorizontal,SimVertical,SimDiagonal1,SimDiagonal2]=run_c8(img,n,m,fname)

SimHorizontal = zeros([n m]);
SimVertical = zeros([n m]);
SimDiagonal1= zeros([n m]); % top left- to down right
SimDiagonal2= zeros([n m]); % down left - to top right

for i = 1 : n-1,
	j=1;
	SimHorizontal(i,j) = feval(fname,img,i,j,i,j+1);
	SimVertical(i,j) = feval(fname,img,i,j,i+1,j);
	SimDiagonal1(i,j) = feval(fname,img,i,j,i+1,j+1);
	for j = 2 : m-1, % SimDiagonal2(i,j) starts by j=2 therefore j is split into 1 and 2:m-1
		SimHorizontal(i,j) = feval(fname,img,i,j,i,j+1);
		SimVertical(i,j) = feval(fname,img,i,j,i+1,j);
		SimDiagonal1(i,j) = feval(fname,img,i,j,i+1,j+1);
		SimDiagonal2(i,j) = feval(fname,img,i,j,i+1,j-1);
		
	end
	SimVertical(i,m) = feval(fname,img,i,m,i+1,m);
	SimDiagonal2(i,m) = feval(fname,img,i,j,i+1,j-1);
end

for j = 1 : m-1,
  SimHorizontal(n,j) = feval(fname,img,n,j,n,j+1);
end  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% run c4
function [SimHorizontal,SimVertical]=run_c4(img,n,m,fname)

SimHorizontal = zeros([n m]);
SimVertical = zeros([n m]);

for i = 1 : n-1,
  for j = 1 : m-1,
    SimHorizontal(i,j) = feval(fname,img,i,j,i,j+1);
    SimVertical(i,j) = feval(fname,img,i,j,i+1,j);
  end
  SimVertical(i,m) = feval(fname,img,i,m,i+1,m);
end

for j = 1 : m-1,
  SimHorizontal(n,j) = feval(fname,img,n,j,n,j+1);
end  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Similarity=inner_sim_gray(img,i1,j1,i2,j2,typ)
Dist = img(i1,j1)-img(i2,j2);
Similarity = 1/(0.001 + Dist*Dist);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Similarity=inner_sim_col(img,i1,j1,i2,j2,typ)
Dist = sum(abs(img(i1,j1,:)-img(i2,j2,:)));
Similarity = 1/(0.001 + Dist*Dist);



